/*
* $LastChangedBy$
* $LastChangedDate$
* $LastChangedRevision$
* $HeadURL$
*
* Copyright 2014 BlueConic Inc./BlueConic B.V. All rights reserved.
*/

import Foundation

struct BlueConicCache: Codable {
    var properties: [String: [String]]
    var domainGroup: String?
    var modified: Bool = false

    enum CacheKeys: String, CodingKey {
        case properties
        case domainGroup
    }

    /**
    Default init() for BlueConicCache
    */
    init() {
        self.properties = [String: [String]]()
        self.modified = true
    }


    /**
    Returns the domain group.

    - returns: the domain group.
    */
     func getDomainGroup() -> String? {
        return self.domainGroup
    }

    /**
    Setter for the domain group.
    - parameter domainGroup: The domain group to set.
    */
    mutating func setDomainGroup(_ domainGroup: String) {
        self.domainGroup = domainGroup
        setLastUpdate()
    }

    /**
    Returns the first value for a given profile property.

    - parameter property: The name of the profile property
    - returns: The first value or empty if no value was present.
    */
     func getProfileValue(_ property: String) -> String {
        if let matches = self.properties[property], matches.count > 0 {
            return matches[0]
        }
        return ""
    }

    /**
    Returns the values for a given profile property.

    - parameter property: The name of the profile property
    - returns: A collection containing the values.
    */
     func getProfileValues(_ property: String) -> [String]? {
        return self.properties[property]
    }

    /**
    Returns all values currently in this local cache.

    - returns: The dictionary mapping propery name String on the corresponding array of String values.
    */
     func getProperties() -> [String: [String]] {
        return self.properties
    }

    /**
    Adds property values to the profile cache. The values from the collection are added to the profile.

    - parameter name: the profile property to add the values for
    - parameter values: The property values to add to the profile cache.
    */
    mutating func addProperties(_ name: String, values: [String]) {
        // Check if name is empty/ values may be an empty array
        if name == "" {
            // Do nothing
            return
        }

        // Check whether properties exist with this name
        var newValues = [String]()
        if let existing = self.properties[name], existing.count > 0 {
            newValues = existing
        }

        // Add the new values to the array
        for value in values {
            if !newValues.contains(value) {
                newValues.append(value)
            }
        }

        // Create immutable objects to store in the Cache
        let immutableValue = newValues

        // Update the Cache
        self.properties[name] = immutableValue

        // Set the modified flag
        self.setLastUpdate()
    }

    /**
    Sets property values in the profile cache, existing values will be removed.

    - parameter name: the profile property to add the values for
    - parameter values: The property values to set for the profile cache.
    */
    mutating func setProperties(_ name: String, values: [String]) {
        // Check if name is empty/ values may be an empty array
        if name == "" {
            // Do nothing
            return
        }

        self.properties[name] = BlueConicCache.filterEmpty(values)

        self.setLastUpdate()
    }

    /**
    Sets property values in the profile cache, existing values will be removed.
    - parameter properties: The new property values to set for the profile cache. This dictionary maps Strings on String arrays.
    */
    mutating func setProperties(_ properties: [String: [String]]) {
        for (key, value) in properties {
            setProperties(key as String, values: value)
        }
    }
    /**
     Filter out the empty strings in the array.
     - parameter values: A list of values.
     - returns: A Filtered list without empty values.
    */
     static func filterEmpty(_ values: [String]) -> [String] {
        if values.count == 0 {
            return []
        }

        var result = [String]()
        for value in values where value != "" {
            result.append(value)
        }
        return result
    }

    /**
    Clears all entries from this local cache.
    */
    mutating func clear() {
        self.properties.removeAll()
        self.setLastUpdate()
    }

    /**
     Get the hash from all cached properties.
     - returns: A hashed string of all the properties.
    */
     func getHash() -> String {
        return ProfileHash.getHash(self.properties)
    }

    /**
    Set the modified flag.
    */
    mutating func setLastUpdate() {
        self.modified = true
    }

    /**
    Reads the modified flag and resets it.
    - returns: The value of the modified flag before it was reset.
    */
    mutating func checkModified() -> Bool {
        let result = self.modified
        self.modified = false
        return result
    }
}
